using ILoan.Tools.Infrastructure.Services;

namespace ILoan.Tools.Application.Pages;

public partial class CoreRules
{
    [Inject]
    protected IJSRuntime JSRuntime { get; set; }

    [Inject]
    protected NavigationManager NavigationManager { get; set; }

    [Inject]
    protected DialogService DialogService { get; set; }

    [Inject]
    protected TooltipService TooltipService { get; set; }

    [Inject]
    protected ContextMenuService ContextMenuService { get; set; }

    [Inject]
    protected NotificationService NotificationService { get; set; }

    [Inject]
    public RulesService RulesService { get; set; }

    [Inject]
    public RuleFileGeneratorService RuleFileGeneratorService { get; set; }

    protected IEnumerable<CoreRule> coreRules;

    protected RadzenDataGrid<CoreRule> grid0;
    protected override async Task OnInitializedAsync()
    {
        coreRules = await RulesService.GetCoreRules();
    }

    protected async Task AddButtonClick(MouseEventArgs args)
    {
        await DialogService.OpenAsync<AddCoreRule>("Add CoreRule", null);
        await grid0.Reload();
    }

    /// <summary>
    /// Execute RuleFileGeneratorService to generate SQL files
    /// </summary>
    /// <param name="args"></param>
    /// <returns></returns>
    protected async Task GenerateButtonClick()
    {
        await RuleFileGeneratorService.GenerateRuleFilesAsync();
        NotificationService.Notify(new NotificationMessage
        {
            Severity = NotificationSeverity.Success,
            Summary = $"Success",
            Detail = $"Rule files generated successfully",

        });
    }

    protected async Task EditRow(CoreRule args)
    {
        await DialogService.OpenAsync<EditCoreRule>("Edit CoreRule", new Dictionary<string, object> { {"ID", args.ID} });
    }

    protected async Task GridDeleteButtonClick(MouseEventArgs args, CoreRule coreRule)
    {
        try
        {
            if (await DialogService.Confirm("Are you sure you want to delete this record?") == true)
            {
                var deleteResult = await RulesService.DeleteCoreRule(coreRule.ID);

                if (deleteResult != null)
                {
                    await grid0.Reload();
                }
            }
        }
        catch (Exception ex)
        {
            NotificationService.Notify(new NotificationMessage
            {
                Severity = NotificationSeverity.Error,
                Summary = $"Error",
                Detail = $"Unable to delete CoreRule"
            });
        }
    }

    protected CoreRule coreRuleChild;
    protected async Task GetChildData(CoreRule args)
    {
        coreRuleChild = args;
        var CoreRuleCriteriaResult = await RulesService.GetCoreRuleCriteria(new Query { Filter = $@"i => i.RuleID == {args.ID}", Expand = "CoreRule" });
        if (CoreRuleCriteriaResult != null)
        {
            args.CoreRuleCriteria = CoreRuleCriteriaResult.ToList();
        }
        var CoreRuleCriteriaOperatorsResult = await RulesService.GetCoreRuleCriteriaOperators(new Query { Filter = $@"i => i.RuleID == {args.ID}", Expand = "CoreRule" });
        if (CoreRuleCriteriaOperatorsResult != null)
        {
            args.CoreRuleCriteriaOperators = CoreRuleCriteriaOperatorsResult.ToList();
        }
    }
    protected CoreRuleCriterion coreRuleCriterionCoreRuleCriteria;

    protected IEnumerable<CoreRule> coreRulesForRuleIDCoreRuleCriteria;

    protected RadzenDataGrid<CoreRuleCriterion> CoreRuleCriteriaDataGrid;

    protected async Task CoreRuleCriteriaAddButtonClick(MouseEventArgs args, CoreRule data)
    {

        var dialogResult = await DialogService.OpenAsync<AddCoreRuleCriterion>("Add CoreRuleCriteria", new Dictionary<string, object> { {"RuleID" , data.ID} }, new DialogOptions
        {
            Width = "70%"
        });

        await GetChildData(data);
        await CoreRuleCriteriaDataGrid.Reload();

    }

    protected async Task CoreRuleCriteriaRowSelect(CoreRuleCriterion args, CoreRule data)
    {
        var dialogResult = await DialogService.OpenAsync<EditCoreRuleCriterion>("Edit CoreRuleCriteria", new Dictionary<string, object> { {"ID", args.ID} });
        await GetChildData(data);
        await CoreRuleCriteriaDataGrid.Reload();
    }

    protected async Task CoreRuleCriteriaDeleteButtonClick(MouseEventArgs args, CoreRuleCriterion coreRuleCriterion)
    {
        try
        {
            if (await DialogService.Confirm("Are you sure you want to delete this record?") == true)
            {
                var deleteResult = await RulesService.DeleteCoreRuleCriterion(coreRuleCriterion.ID);

                await GetChildData(coreRuleChild);

                if (deleteResult != null)
                {
                    await CoreRuleCriteriaDataGrid.Reload();
                }
            }
        }
        catch (System.Exception ex)
        {
            NotificationService.Notify(new NotificationMessage
            {
                Severity = NotificationSeverity.Error,
                Summary = $"Error",
                Detail = $"Unable to delete CoreRuleCriterion"
            });
        }
    }
    protected CoreRuleCriteriaOperator coreRuleCriteriaOperatorCoreRuleCriteriaOperators;

    protected IEnumerable<CoreRule> coreRulesForRuleIDCoreRuleCriteriaOperators;

    protected RadzenDataGrid<CoreRuleCriteriaOperator> CoreRuleCriteriaOperatorsDataGrid;

    protected async Task CoreRuleCriteriaOperatorsAddButtonClick(MouseEventArgs args, CoreRule data)
    {

        var dialogResult = await DialogService.OpenAsync<AddCoreRuleCriteriaOperator>("Add CoreRuleCriteriaOperators", new Dictionary<string, object> { {"RuleID" , data.ID} });
        await GetChildData(data);
        await CoreRuleCriteriaOperatorsDataGrid.Reload();

    }

    protected async Task CoreRuleCriteriaOperatorsRowSelect(CoreRuleCriteriaOperator args, CoreRule data)
    {
        var dialogResult = await DialogService.OpenAsync<EditCoreRuleCriteriaOperator>("Edit CoreRuleCriteriaOperators", new Dictionary<string, object> { {"ID", args.ID} });
        await GetChildData(data);
        await CoreRuleCriteriaOperatorsDataGrid.Reload();
    }

    protected async Task CoreRuleCriteriaOperatorsDeleteButtonClick(MouseEventArgs args, CoreRuleCriteriaOperator coreRuleCriteriaOperator)
    {
        try
        {
            if (await DialogService.Confirm("Are you sure you want to delete this record?") == true)
            {
                var deleteResult = await RulesService.DeleteCoreRuleCriteriaOperator(coreRuleCriteriaOperator.ID);

                await GetChildData(coreRuleChild);

                if (deleteResult != null)
                {
                    await CoreRuleCriteriaOperatorsDataGrid.Reload();
                }
            }
        }
        catch (System.Exception ex)
        {
            NotificationService.Notify(new NotificationMessage
            {
                Severity = NotificationSeverity.Error,
                Summary = $"Error",
                Detail = $"Unable to delete CoreRuleCriteriaOperator"
            });
        }
    }
}